@@ -1,3 +1,25 @@
+// Copyright (C) 2004-2025 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
 #include "mupdf/fitz.h"
 
 #include "draw-imp.h"
@@ -435,11 +457,9 @@ fz_blend_nonseparable_gray(byte * FZ_RESTRICT bp, int bal, const byte * FZ_RESTR
 				int saba = fz_mul255(sa, ba);
 
 				/* ugh, division to get non-premul components */
-				int invsa = sa ? 255 * 256 / sa : 0;
-				int invba = ba ? 255 * 256 / ba : 0;
+				int invsa = 255 * 256 / sa;
+				int invba = 255 * 256 / ba;
 				int k;
-				int sg = (sp[0] * invsa) >> 8;
-				int bg = (bp[0] * invba) >> 8;
 
 				switch (blendmode)
 				{
@@ -447,12 +467,18 @@ fz_blend_nonseparable_gray(byte * FZ_RESTRICT bp, int bal, const byte * FZ_RESTR
 				case FZ_BLEND_HUE:
 				case FZ_BLEND_SATURATION:
 				case FZ_BLEND_COLOR:
-					bp[0] = bal ? fz_mul255(bp[n], bg) : bg;
+				{
+					int bg = (bp[0] * invba) >> 8;
+					bp[0] = fz_mul255(255 - sa, bp[0]) + fz_mul255(255 - ba, sp[0]) + fz_mul255(saba, bg);
 					break;
+				}
 				case FZ_BLEND_LUMINOSITY:
-					bp[0] = bal ? fz_mul255(bp[n], sg) : sg;
+				{
+					int sg = (sp[0] * invsa) >> 8;
+					bp[0] = fz_mul255(255 - sa, bp[0]) + fz_mul255(255 - ba, sp[0]) + fz_mul255(saba, sg);
 					break;
 				}
+				}
 
 				/* Normal blend for spots */
 				for (k = first_spot; k < n; k++)
@@ -493,7 +519,7 @@ fz_blend_nonseparable(byte * FZ_RESTRICT bp, int bal, const byte * FZ_RESTRICT s
 				int saba = fz_mul255(sa, ba);
 
 				/* ugh, division to get non-premul components */
-				int invsa = sa ? 255 * 256 / sa : 0;
+				int invsa = 255 * 256 / sa;
 				int invba = 255 * 256 / ba;
 
 				int sr = (sp[0] * invsa) >> 8;
@@ -535,15 +561,12 @@ fz_blend_nonseparable(byte * FZ_RESTRICT bp, int bal, const byte * FZ_RESTRICT s
 				/* CMYK */
 				if (complement)
 				{
-					int sk = (sp[3] * invsa) >> 8;
-					int bk = (bp[3] * invba) >> 8;
-
 					rr = 255 - rr;
 					rg = 255 - rg;
 					rb = 255 - rb;
-					bp[0] = fz_mul255(255 - sa, 255 - bp[0]) + fz_mul255(255 - ba, sp[0]) + fz_mul255(saba, rr);
-					bp[1] = fz_mul255(255 - sa, 255 - bp[1]) + fz_mul255(255 - ba, sp[1]) + fz_mul255(saba, rg);
-					bp[2] = fz_mul255(255 - sa, 255 - bp[2]) + fz_mul255(255 - ba, sp[2]) + fz_mul255(saba, rb);
+					bp[0] = fz_mul255(255 - sa, bp[0]) + fz_mul255(255 - ba, sp[0]) + fz_mul255(saba, rr);
+					bp[1] = fz_mul255(255 - sa, bp[1]) + fz_mul255(255 - ba, sp[1]) + fz_mul255(saba, rg);
+					bp[2] = fz_mul255(255 - sa, bp[2]) + fz_mul255(255 - ba, sp[2]) + fz_mul255(saba, rb);
 
 					switch (blendmode)
 					{
@@ -551,12 +574,13 @@ fz_blend_nonseparable(byte * FZ_RESTRICT bp, int bal, const byte * FZ_RESTRICT s
 					case FZ_BLEND_HUE:
 					case FZ_BLEND_SATURATION:
 					case FZ_BLEND_COLOR:
-						bp[3] = bal ? fz_mul255(bp[n], bk) : bk;
+						k = (bp[3] * invba) >> 8;
 						break;
 					case FZ_BLEND_LUMINOSITY:
-						bp[3] = bal ? fz_mul255(bp[n], sk) : sk;
+						k = (sp[3] * invsa) >> 8;
 						break;
 					}
+					bp[3] = fz_mul255(255 - sa, bp[3]) + fz_mul255(255 - ba, sp[3]) + fz_mul255(saba, k);
 				}
 				else
 				{
@@ -1101,17 +1125,16 @@ fz_blend_pixmap(fz_context *ctx, fz_pixmap * FZ_RESTRICT dst, fz_pixmap * FZ_RES
 
 	x = bbox.x0;
 	y = bbox.y0;
-	w = bbox.x1 - bbox.x0;
-	h = bbox.y1 - bbox.y0;
-
+	w = fz_irect_width(bbox);
+	h = fz_irect_height(bbox);
 	if (w == 0 || h == 0)
 		return;
 
 	complement = fz_colorspace_is_subtractive(ctx, src->colorspace);
 	n = src->n;
-	sp = src->samples + (unsigned int)((y - src->y) * src->stride + (x - src->x) * src->n);
+	sp = src->samples + (y - src->y) * (size_t)src->stride + (x - src->x) * (size_t)src->n;
 	sa = src->alpha;
-	dp = dst->samples + (unsigned int)((y - dst->y) * dst->stride + (x - dst->x) * dst->n);
+	dp = dst->samples + (y - dst->y) * (size_t)dst->stride + (x - dst->x) * (size_t)dst->n;
 	da = dst->alpha;
 
 	if (n == 1)
@@ -1129,7 +1152,7 @@ fz_blend_pixmap(fz_context *ctx, fz_pixmap * FZ_RESTRICT dst, fz_pixmap * FZ_RES
 
 	if (!isolated)
 	{
-		const unsigned char *hp = shape->samples + (unsigned int)((y - shape->y) * shape->stride + (x - shape->x));
+		const unsigned char *hp = shape->samples + (y - shape->y) * (size_t)shape->stride + (x - shape->x);
 
 		while (h--)
 		{
@@ -1309,18 +1332,17 @@ fz_blend_pixmap_knockout(fz_context *ctx, fz_pixmap * FZ_RESTRICT dst, fz_pixmap
 
 	x = bbox.x0;
 	y = bbox.y0;
-	w = bbox.x1 - bbox.x0;
-	h = bbox.y1 - bbox.y0;
-
+	w = fz_irect_width(bbox);
+	h = fz_irect_height(bbox);
 	if (w == 0 || h == 0)
 		return;
 
 	n = src->n;
-	sp = src->samples + (unsigned int)((y - src->y) * src->stride + (x - src->x) * src->n);
+	sp = src->samples + (y - src->y) * (size_t)src->stride + (x - src->x) * (size_t)src->n;
 	sa = src->alpha;
-	dp = dst->samples + (unsigned int)((y - dst->y) * dst->stride + (x - dst->x) * dst->n);
+	dp = dst->samples + (y - dst->y) * (size_t)dst->stride + (x - dst->x) * (size_t)dst->n;
 	da = dst->alpha;
-	hp = shape->samples + (unsigned int)((y - shape->y) * shape->stride + (x - shape->x));
+	hp = shape->samples + (y - shape->y) * (size_t)shape->stride + (x - shape->x);
 
 #ifdef PARANOID_PREMULTIPLY
 	if (sa)
