@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2022 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -80,6 +80,8 @@ FONT(sil,	CharisSIL_BoldItalic_cff,	"Charis SIL",		ANY_SCRIPT,	FZ_LANG_UNSET,	NO
 #endif
 
 #ifndef TOFU_NOTO
+FONT(urw,	NimbusBoxes_Regular_cff,	"Nimbus Boxes",		ANY_SCRIPT,	FZ_LANG_UNSET,	NO_SUBFONT,	REGULAR)
+
 FONT(noto,	NotoSerif_Regular_otf,	"Noto Serif",	UCDN_SCRIPT_LATIN,	FZ_LANG_UNSET,	NO_SUBFONT,	REGULAR)
 ALIAS(noto,	NotoSerif_Regular_otf,	"Noto Serif",	UCDN_SCRIPT_GREEK,	FZ_LANG_UNSET,	NO_SUBFONT,	REGULAR)
 ALIAS(noto,	NotoSerif_Regular_otf,	"Noto Serif",	UCDN_SCRIPT_CYRILLIC,	FZ_LANG_UNSET,	NO_SUBFONT,	REGULAR)
@@ -113,11 +115,14 @@ DEFNOTO(NotoSansCaucasianAlbanian_Regular_otf,		"Noto Sans Caucasian Albanian",
 DEFNOTO(NotoSansChakma_Regular_otf,			"Noto Sans Chakma",			UCDN_SCRIPT_CHAKMA)
 DEFNOTO(NotoSansCham_Regular_otf,			"Noto Sans Cham",			UCDN_SCRIPT_CHAM)
 DEFNOTO(NotoSansCherokee_Regular_otf,			"Noto Sans Cherokee",			UCDN_SCRIPT_CHEROKEE)
+DEFNOTO(NotoSansChorasmian_Regular_otf,			"Noto Sans Chorasmian",			UCDN_SCRIPT_CHORASMIAN)
 DEFNOTO(NotoSansCoptic_Regular_otf,			"Noto Sans Coptic",			UCDN_SCRIPT_COPTIC)
 DEFNOTO(NotoSansCuneiform_Regular_otf,			"Noto Sans Cuneiform",			UCDN_SCRIPT_CUNEIFORM)
 DEFNOTO(NotoSansCypriot_Regular_otf,			"Noto Sans Cypriot",			UCDN_SCRIPT_CYPRIOT)
+DEFNOTO(NotoSansCyproMinoan_Regular_otf,		"Noto Sans Cypro Minoan",		UCDN_SCRIPT_CYPRO_MINOAN)
 DEFNOTO(NotoSansDeseret_Regular_otf,			"Noto Sans Deseret",			UCDN_SCRIPT_DESERET)
 DEFNOTO(NotoSerifDevanagari_Regular_otf,		"Noto Serif Devanagari",		UCDN_SCRIPT_DEVANAGARI)
+DEFNOTO(NotoSerifDivesAkuru_Regular_otf,		"Noto Serif Dives Akuru",		UCDN_SCRIPT_DIVES_AKURU)
 DEFNOTO(NotoSerifDogra_Regular_otf,			"Noto Serif Dogra",			UCDN_SCRIPT_DOGRA)
 DEFNOTO(NotoSansDuployan_Regular_otf,			"Noto Sans Duployan",			UCDN_SCRIPT_DUPLOYAN)
 DEFNOTO(NotoSansEgyptianHieroglyphs_Regular_otf,	"Noto Sans Egyptian Hieroglyphs",	UCDN_SCRIPT_EGYPTIAN_HIEROGLYPHS)
@@ -142,7 +147,9 @@ DEFNOTO(NotoSansJavanese_Regular_otf,			"Noto Sans Javanese",			UCDN_SCRIPT_JAVA
 DEFNOTO(NotoSansKaithi_Regular_otf,			"Noto Sans Kaithi",			UCDN_SCRIPT_KAITHI)
 DEFNOTO(NotoSerifKannada_Regular_otf,			"Noto Serif Kannada",			UCDN_SCRIPT_KANNADA)
 DEFNOTO(NotoSansKayahLi_Regular_otf,			"Noto Sans Kayah Li",			UCDN_SCRIPT_KAYAH_LI)
+DEFNOTO(NotoSansKawi_Regular_otf,			"Noto Sans Kawi",			UCDN_SCRIPT_KAWI)
 DEFNOTO(NotoSansKharoshthi_Regular_otf,			"Noto Sans Kharoshthi",			UCDN_SCRIPT_KHAROSHTHI)
+DEFNOTO(NotoSerifKhitanSmallScript_Regular_otf,		"Noto Serif Khitan Small Script",	UCDN_SCRIPT_KHITAN_SMALL_SCRIPT)
 DEFNOTO(NotoSerifKhmer_Regular_otf,			"Noto Serif Khmer",			UCDN_SCRIPT_KHMER)
 DEFNOTO(NotoSerifKhojki_Regular_otf,			"Noto Serif Khojki",			UCDN_SCRIPT_KHOJKI)
 DEFNOTO(NotoSansKhudawadi_Regular_otf,			"Noto Sans Khudawadi",			UCDN_SCRIPT_KHUDAWADI)
@@ -155,6 +162,7 @@ DEFNOTO(NotoSansLisu_Regular_otf,			"Noto Sans Lisu",			UCDN_SCRIPT_LISU)
 DEFNOTO(NotoSansLycian_Regular_otf,			"Noto Sans Lycian",			UCDN_SCRIPT_LYCIAN)
 DEFNOTO(NotoSansLydian_Regular_otf,			"Noto Sans Lydian",			UCDN_SCRIPT_LYDIAN)
 DEFNOTO(NotoSansMahajani_Regular_otf,			"Noto Sans Mahajani",			UCDN_SCRIPT_MAHAJANI)
+DEFNOTO(NotoSerifMakasar_Regular_otf,			"Noto Serif Makasar",			UCDN_SCRIPT_MAKASAR)
 DEFNOTO(NotoSerifMalayalam_Regular_otf,			"Noto Serif Malayalam",			UCDN_SCRIPT_MALAYALAM)
 DEFNOTO(NotoSansMandaic_Regular_otf,			"Noto Sans Mandaic",			UCDN_SCRIPT_MANDAIC)
 DEFNOTO(NotoSansManichaean_Regular_otf,			"Noto Sans Manichaean",			UCDN_SCRIPT_MANICHAEAN)
@@ -172,6 +180,8 @@ DEFNOTO(NotoSansMro_Regular_otf,			"Noto Sans Mro",			UCDN_SCRIPT_MRO)
 DEFNOTO(NotoSansMultani_Regular_otf,			"Noto Sans Multani",			UCDN_SCRIPT_MULTANI)
 DEFNOTO(NotoSerifMyanmar_Regular_otf,			"Noto Serif Myanmar",			UCDN_SCRIPT_MYANMAR)
 DEFNOTO(NotoSansNabataean_Regular_otf,			"Noto Sans Nabataean",			UCDN_SCRIPT_NABATAEAN)
+DEFNOTO(NotoSansNagMundari_Regular_otf,			"Noto Sans Nag Mundari",		UCDN_SCRIPT_NAG_MUNDARI)
+DEFNOTO(NotoSansNandinagari_Regular_otf,		"Noto Sans Nandinagari",		UCDN_SCRIPT_NANDINAGARI)
 DEFNOTO(NotoSansNewa_Regular_otf,			"Noto Sans Newa",			UCDN_SCRIPT_NEWA)
 DEFNOTO(NotoSansNewTaiLue_Regular_otf,			"Noto Sans New Tai Lue",		UCDN_SCRIPT_NEW_TAI_LUE)
 DEFNOTO(NotoSansNKo_Regular_otf,			"Noto Sans N Ko",			UCDN_SCRIPT_NKO)
@@ -186,8 +196,9 @@ DEFNOTO(NotoSansOldPersian_Regular_otf,			"Noto Sans Old Persian",		UCDN_SCRIPT_
 DEFNOTO(NotoSansOldSogdian_Regular_otf,			"Noto Sans Old Sogdian",		UCDN_SCRIPT_OLD_SOGDIAN)
 DEFNOTO(NotoSansOldSouthArabian_Regular_otf,		"Noto Sans Old South Arabian",		UCDN_SCRIPT_OLD_SOUTH_ARABIAN)
 DEFNOTO(NotoSansOldTurkic_Regular_otf,			"Noto Sans Old Turkic",			UCDN_SCRIPT_OLD_TURKIC)
+DEFNOTO(NotoSerifOldUyghur_Regular_otf,			"Noto Serif Old Uyghur",		UCDN_SCRIPT_OLD_UYGHUR)
 DEFNOTO(NotoSansOlChiki_Regular_otf,			"Noto Sans Ol Chiki",			UCDN_SCRIPT_OL_CHIKI)
-DEFNOTO(NotoSansOriya_Regular_otf,			"Noto Sans Oriya",			UCDN_SCRIPT_ORIYA)
+DEFNOTO(NotoSerifOriya_Regular_otf,			"Noto Serif Oriya",			UCDN_SCRIPT_ORIYA)
 DEFNOTO(NotoSansOsage_Regular_otf,			"Noto Sans Osage",			UCDN_SCRIPT_OSAGE)
 DEFNOTO(NotoSansOsmanya_Regular_otf,			"Noto Sans Osmanya",			UCDN_SCRIPT_OSMANYA)
 DEFNOTO(NotoSansPahawhHmong_Regular_otf,		"Noto Sans Pahawh Hmong",		UCDN_SCRIPT_PAHAWH_HMONG)
@@ -216,6 +227,7 @@ DEFNOTO(NotoSansTaiLe_Regular_otf,			"Noto Sans TaiLe",			UCDN_SCRIPT_TAI_LE)
 DEFNOTO(NotoSansTaiTham_Regular_otf,			"Noto Sans Tai Tham",			UCDN_SCRIPT_TAI_THAM)
 DEFNOTO(NotoSansTaiViet_Regular_otf,			"Noto Sans Tai Viet",			UCDN_SCRIPT_TAI_VIET)
 DEFNOTO(NotoSansTakri_Regular_otf,			"Noto Sans Takri",			UCDN_SCRIPT_TAKRI)
+DEFNOTO(NotoSansTangsa_Regular_otf,			"Noto Sans Tangsa",			UCDN_SCRIPT_TANGSA)
 DEFNOTO(NotoSerifTamil_Regular_otf,			"Noto Serif Tamil",			UCDN_SCRIPT_TAMIL)
 DEFNOTO(NotoSerifTelugu_Regular_otf,			"Noto Serif Telugu",			UCDN_SCRIPT_TELUGU)
 DEFNOTO(NotoSansThaana_Regular_otf,			"Noto Sans Thaana",			UCDN_SCRIPT_THAANA)
@@ -223,8 +235,10 @@ DEFNOTO(NotoSerifThai_Regular_otf,			"Noto Serif Thai",			UCDN_SCRIPT_THAI)
 DEFNOTO(NotoSerifTibetan_Regular_otf,			"Noto Serif Tibetan",			UCDN_SCRIPT_TIBETAN)
 DEFNOTO(NotoSansTifinagh_Regular_otf,			"Noto Sans Tifinagh",			UCDN_SCRIPT_TIFINAGH)
 DEFNOTO(NotoSansTirhuta_Regular_otf,			"Noto Sans Tirhuta",			UCDN_SCRIPT_TIRHUTA)
+DEFNOTO(NotoSerifToto_Regular_otf,			"Noto Serif Toto",			UCDN_SCRIPT_TOTO)
 DEFNOTO(NotoSansUgaritic_Regular_otf,			"Noto Sans Ugaritic",			UCDN_SCRIPT_UGARITIC)
 DEFNOTO(NotoSansVai_Regular_otf,			"Noto Sans Vai",			UCDN_SCRIPT_VAI)
+DEFNOTO(NotoSerifVithkuqi_Regular_otf,			"Noto Serif Vithukqi",			UCDN_SCRIPT_VITHKUQI)
 DEFNOTO(NotoSansWancho_Regular_otf,			"Noto Sans Wancho",			UCDN_SCRIPT_WANCHO)
 DEFNOTO(NotoSansWarangCiti_Regular_otf,			"Noto Sans Warang Citi",		UCDN_SCRIPT_WARANG_CITI)
 DEFNOTO(NotoSerifYezidi_Regular_otf,			"Noto Serif Yezidi",			UCDN_SCRIPT_YEZIDI)
@@ -235,21 +249,21 @@ DEFNOTO(NotoSansZanabazarSquare_Regular_otf,		"Noto Sans Zanabazar Square",		UCD
 DEFNOTO(NotoSerifTangut_Regular_otf,			"Noto Serif Tangut",			UCDN_SCRIPT_TANGUT)
 #endif
 
+#if NOTO_SIGNWRITING
+DEFNOTO(NotoSansSignWriting_Regular_otf,		"Noto Sans SignWriting",		UCDN_SCRIPT_SIGNWRITING)
+#endif
+
 #undef DEFNOTO
 
 #define DEFNOTO(symbol, name, script) EMPTY(script)
 
-DEFNOTO(NotoSansChorasmian_Regular_otf,			"Noto Sans Chorasmian",			UCDN_SCRIPT_CHORASMIAN)
-DEFNOTO(NotoSansCyproMinoan_Regular_otf,		"Noto Sans Cypro Minoan",		UCDN_SCRIPT_CYPRO_MINOAN)
-DEFNOTO(NotoSansDivesAkuru_Regular_otf,			"Noto Sans Dives Akuru",		UCDN_SCRIPT_DIVES_AKURU)
-DEFNOTO(NotoSansKhitanSmallScript_Regular_otf,		"Noto Sans Khitan Small Script",	UCDN_SCRIPT_KHITAN_SMALL_SCRIPT)
-DEFNOTO(NotoSansMakasar_Regular_otf,			"Noto Sans Makasar",			UCDN_SCRIPT_MAKASAR)
-DEFNOTO(NotoSansNandinagari_Regular_otf,		"Noto Sans Nandinagari",		UCDN_SCRIPT_NANDINAGARI)
-DEFNOTO(NotoSansOldUyghur_Regular_otf,			"Noto Sans Old Uyghur",			UCDN_SCRIPT_OLD_UYGHUR)
-DEFNOTO(NotoSansToto_Regular_otf,			"Noto Sans Toto",			UCDN_SCRIPT_TOTO)
-DEFNOTO(NotoSansVithkuqi_Regular_otf,			"Noto Sans Vithukqi",			UCDN_SCRIPT_VITHKUQI)
-DEFNOTO(NotoSansSignwriting_Regular_otf,		"Noto Sans Signwriting",		UCDN_SCRIPT_SIGNWRITING)
-DEFNOTO(NotoSansTangsa_Regular_otf,			"Noto Sans Tangsa",			UCDN_SCRIPT_TANGSA)
+DEFNOTO(NotoSansTodhri_Regular_otf,			"Noto Sans Todhri",			UCDN_SCRIPT_TODHRI)
+DEFNOTO(NotoSansGaray_Regular_otf,			"Noto Sans Garay",			UCDN_SCRIPT_GARAY)
+DEFNOTO(NotoSansTuluTigalari_Regular_otf,		"Noto Sans Tulu Tigalari",		UCDN_SCRIPT_TULU_TIGALARI)
+DEFNOTO(NotoSansSunuwar_Regular_otf,			"Noto Sans Sunuwar",			UCDN_SCRIPT_SUNUWAR)
+DEFNOTO(NotoSansGurungKhema_Regular_otf,		"Noto Sans Gurung Khema",		UCDN_SCRIPT_GURUNG_KHEMA)
+DEFNOTO(NotoSansKiratRai_Regular_otf,			"Noto Sans Kirat Rai",			UCDN_SCRIPT_KIRAT_RAI)
+DEFNOTO(NotoSansOlOnal_Regular_otf,			"Noto Sans Ol Onal",			UCDN_SCRIPT_OL_ONAL)
 
 #undef DEFNOTO
 
