@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2022 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -97,7 +97,7 @@ typedef struct
 	size_t len;
 
 	int symbolic;
-	int cidfont;
+	int is_cidfont;
 
 	uint8_t major;
 	uint8_t minor;
@@ -112,8 +112,8 @@ typedef struct
 	index_t charstrings_index;
 	index_t local_index;
 	index_t fdarray_index;
-	int gsubr_bias;
-	int subr_bias;
+	uint16_t gsubr_bias;
+	uint16_t subr_bias;
 	uint32_t top_dict_index_offset;
 	uint32_t string_index_offset;
 	uint32_t global_index_offset;
@@ -128,6 +128,7 @@ typedef struct
 	uint32_t fdselect_offset;
 	uint32_t fdselect_len;
 	uint32_t fdarray_index_offset;
+	uint32_t charstring_type;
 
 	uint16_t unpacked_charset_len;
 	uint16_t unpacked_charset_max;
@@ -136,9 +137,15 @@ typedef struct
 	struct
 	{
 		fz_buffer *rewritten_dict;
+		fz_buffer *rewritten_private;
 		uint32_t offset;
 		uint32_t len;
 		uint32_t fixup;
+		uint32_t local_index_offset;
+		index_t local_index;
+		usage_list_t local_usage;
+		uint16_t subr_bias;
+		fz_buffer *local_subset;
 	} *fdarray;
 
 	struct
@@ -163,6 +170,7 @@ typedef struct
 	usage_list_t extra_gids_to_keep;
 
 	uint16_t *gid_to_cid;
+	uint8_t *gid_to_font;
 } cff_t;
 
 /* cid -> gid */
@@ -250,6 +258,19 @@ offsize_for_offset(uint32_t offset)
 	return 4;
 }
 
+uint16_t
+subr_bias(fz_context *ctx, cff_t *cff, uint16_t count)
+{
+	if (cff->charstring_type == 1)
+		return 0;
+	else if (count < 1240)
+		return 107;
+	else if (count < 33900)
+		return 1131;
+	else
+		return 32768;
+}
+
 /* Index functions */
 
 /* "Load" an index and check it for plausibility (no overflows etc) */
@@ -709,7 +730,7 @@ dict_write_args(fz_context *ctx, fz_output *out, dict_iterator *di)
 }
 
 static void
-do_subset(fz_context *ctx, cff_t *cff, fz_buffer **buffer, usage_list_t *keep_list, index_t *index)
+do_subset(fz_context *ctx, cff_t *cff, fz_buffer **buffer, usage_list_t *keep_list, index_t *index, int keep_notdef)
 {
 	uint8_t *d, *strings;
 	uint32_t i, offset, end;
@@ -734,7 +755,7 @@ do_subset(fz_context *ctx, cff_t *cff, fz_buffer **buffer, usage_list_t *keep_li
 			/* Keep this */
 			gid++;
 		}
-		else if (i == 0)
+		else if (keep_notdef && i == 0)
 		{
 			/* Keep this. */
 		}
@@ -764,10 +785,12 @@ do_subset(fz_context *ctx, cff_t *cff, fz_buffer **buffer, usage_list_t *keep_li
 
 	/* Write out the index header */
 	put16(d, num_charstrings); /* count */
-	put8(d+2, offset_size); /* offset size */
+	d +=2;
+	put8(d, offset_size); /* offset size */
+	d += 1;
+
 
 	/* Now copy the charstrings themselves */
-	d += 3;
 	strings = d + offset_size * (num_charstrings+1) - 1;
 	gid = 0;
 	fill = 1;
@@ -780,7 +803,7 @@ do_subset(fz_context *ctx, cff_t *cff, fz_buffer **buffer, usage_list_t *keep_li
 			/* Keep this */
 			gid++;
 		}
-		else if (i == 0)
+		else if (keep_notdef && i == 0)
 		{
 			/* Keep this */
 		}
@@ -804,19 +827,28 @@ do_subset(fz_context *ctx, cff_t *cff, fz_buffer **buffer, usage_list_t *keep_li
 static void
 subset_charstrings(fz_context *ctx, cff_t *cff)
 {
-	do_subset(ctx, cff, &cff->charstrings_subset, &cff->gids_to_keep, &cff->charstrings_index);
+	do_subset(ctx, cff, &cff->charstrings_subset, &cff->gids_to_keep, &cff->charstrings_index, 1);
 }
 
 static void
 subset_locals(fz_context *ctx, cff_t *cff)
 {
-	do_subset(ctx, cff, &cff->local_subset, &cff->local_usage, &cff->local_index);
+	do_subset(ctx, cff, &cff->local_subset, &cff->local_usage, &cff->local_index, 0);
 }
 
 static void
 subset_globals(fz_context *ctx, cff_t *cff)
 {
-	do_subset(ctx, cff, &cff->global_subset, &cff->global_usage, &cff->global_index);
+	do_subset(ctx, cff, &cff->global_subset, &cff->global_usage, &cff->global_index, 0);
+}
+
+static void
+subset_fdarray_locals(fz_context *ctx, cff_t *cff)
+{
+	uint16_t i, n = cff->fdarray_index.count;
+
+	for (i = 0; i < n; i++)
+		do_subset(ctx, cff, &cff->fdarray[i].local_subset, &cff->fdarray[i].local_usage, &cff->fdarray[i].local_index, 0);
 }
 
 /* Charstring "executing" functions */
@@ -885,11 +917,11 @@ drop_usage_list(fz_context *ctx, usage_list_t *list)
 }
 
 static void
-mark_subr_used(fz_context *ctx, cff_t *cff, int subr, int global)
+mark_subr_used(fz_context *ctx, cff_t *cff, int subr, int global, int local_subr_bias, usage_list_t *local_usage)
 {
-	usage_list_t *list = global ? &cff->global_usage : &cff->local_usage;
+	usage_list_t *list = global ? &cff->global_usage : local_usage;
 
-	subr += global ? cff->gsubr_bias : cff->subr_bias;
+	subr += global ? cff->gsubr_bias : local_subr_bias;
 
 	usage_list_add(ctx, list, subr);
 }
@@ -931,10 +963,10 @@ use_sub_char(fz_context *ctx, cff_t *cff, int code)
 do { if (sp + n > (int)(sizeof(stack)/sizeof(*stack))) fz_throw(ctx, FZ_ERROR_FORMAT, "Stack overflow"); sp += n; } while (0)
 
 static void
-execute_charstring(fz_context *ctx, cff_t *cff, const uint8_t *pc, const uint8_t *end)
+execute_charstring(fz_context *ctx, cff_t *cff, const uint8_t *pc, const uint8_t *end, uint16_t subr_bias, usage_list_t *local_usage)
 {
 	double trans[32] = { 0 };
-	double stack[48];
+	double stack[513];
 	int sp = 0;
 	int stem_hints = 0;
 	uint8_t c;
@@ -1003,7 +1035,7 @@ clear:
 
 		case 10: /* callsubr */
 			ATLEAST(1);
-			mark_subr_used(ctx, cff, stack[sp-1], 0);
+			mark_subr_used(ctx, cff, stack[sp-1], 0, subr_bias, local_usage);
 			sp--;
 			break;
 		case 11: /* return */
@@ -1199,7 +1231,7 @@ overflow:
 			break;
 		case 29: /* callgsubr */
 			ATLEAST(1);
-			mark_subr_used(ctx, cff, stack[sp-1], 1);
+			mark_subr_used(ctx, cff, stack[sp-1], 1, subr_bias, local_usage);
 			sp--;
 			break;
 		case 28: /* shortint */
@@ -1248,15 +1280,44 @@ atleast_fail:
 	fz_throw(ctx, FZ_ERROR_FORMAT, "Insufficient operators on the stack: op=%d", c);
 }
 
+
+usage_list_t *
+get_font_locals(fz_context *ctx, cff_t *cff, int gid, int is_pdf_cidfont, uint16_t *subr_bias)
+{
+	usage_t *gids = cff->gids_to_keep.list;
+	int num_gids = cff->gids_to_keep.len;
+
+	if (is_pdf_cidfont && cff->is_cidfont)
+	{
+		uint8_t font = 0;
+		if (gid < num_gids && gids[gid].num < cff->charstrings_index.count)
+			font = cff->gid_to_font[gids[gid].num];
+		else if (gid == 0)
+			font = cff->gid_to_font[gid];
+		if (font >= cff->fdarray_index.count)
+			font = 0;
+
+		if (subr_bias)
+			*subr_bias = cff->fdarray[font].subr_bias;
+		return &cff->fdarray[font].local_usage;
+	}
+
+	if (subr_bias)
+		*subr_bias = cff->subr_bias;
+	return &cff->local_usage;
+}
+
 static void
-scan_charstrings(fz_context *ctx, cff_t *cff)
+scan_charstrings(fz_context *ctx, cff_t *cff, int is_pdf_cidfont)
 {
 	uint32_t offset, end;
 	int num_charstrings = (int)cff->charstrings_index.count;
-	int i, gid;
+	int i, gid, font;
 	usage_t *gids = cff->gids_to_keep.list;
 	int num_gids = cff->gids_to_keep.len;
 	int changed;
+	uint16_t subr_bias;
+	usage_list_t *local_usage = NULL;
 
 	/* Scan through the charstrings.*/
 	offset = index_get(ctx, &cff->charstrings_index, 0);
@@ -1278,7 +1339,8 @@ scan_charstrings(fz_context *ctx, cff_t *cff)
 			/* Drop this */
 			continue;
 		}
-		execute_charstring(ctx, cff, &cff->base[offset], &cff->base[end]);
+		local_usage = get_font_locals(ctx, cff, gid, is_pdf_cidfont, &subr_bias);
+		execute_charstring(ctx, cff, &cff->base[offset], &cff->base[end], subr_bias, local_usage);
 	}
 
 	/* Now we search the 'extra' ones, the 'subrs' (local) and 'gsubrs' (globals)
@@ -1297,7 +1359,9 @@ scan_charstrings(fz_context *ctx, cff_t *cff)
 			usage_list_add(ctx, &cff->gids_to_keep, gid);
 			offset = index_get(ctx, &cff->charstrings_index, gid);
 			end = index_get(ctx, &cff->charstrings_index, gid+1);
-			execute_charstring(ctx, cff, &cff->base[offset], &cff->base[end]);
+
+			local_usage = get_font_locals(ctx, cff, gid, is_pdf_cidfont, &subr_bias);
+			execute_charstring(ctx, cff, &cff->base[offset], &cff->base[end], subr_bias, local_usage);
 			changed = 1;
 		}
 
@@ -1310,10 +1374,32 @@ scan_charstrings(fz_context *ctx, cff_t *cff)
 			gid = cff->local_usage.list[i].num;
 			offset = index_get(ctx, &cff->local_index, gid);
 			end = index_get(ctx, &cff->local_index, gid+1);
-			execute_charstring(ctx, cff, &cff->base[offset], &cff->base[end]);
+
+			local_usage = get_font_locals(ctx, cff, gid, is_pdf_cidfont, &subr_bias);
+			execute_charstring(ctx, cff, &cff->base[offset], &cff->base[end], subr_bias, local_usage);
 			changed = 1;
 		}
 
+		/* Now, run through the per-font locals, seeing what per-font locals and globals they call.  */
+		for (font = 0; font < cff->fdarray_index.count; font++)
+		{
+			for (i = 0; i < cff->fdarray[font].local_usage.len; i++)
+			{
+				gid = cff->fdarray[font].local_usage.list[i].num;
+
+				if (cff->fdarray[font].local_usage.list[i].scanned)
+					continue;
+				cff->fdarray[font].local_usage.list[i].scanned = 1;
+				gid = cff->fdarray[font].local_usage.list[i].num;
+				offset = index_get(ctx, &cff->fdarray[font].local_index, gid);
+				end = index_get(ctx, &cff->fdarray[font].local_index, gid+1);
+
+				local_usage = get_font_locals(ctx, cff, gid, is_pdf_cidfont, &subr_bias);
+				execute_charstring(ctx, cff, &cff->base[offset], &cff->base[end], subr_bias, local_usage);
+				changed = 1;
+			}
+		}
+
 		/* Now, run through the globals, seeing what globals they call.  */
 		for (i = 0; i < cff->global_usage.len; i++)
 		{
@@ -1323,7 +1409,9 @@ scan_charstrings(fz_context *ctx, cff_t *cff)
 			gid = cff->global_usage.list[i].num;
 			offset = index_get(ctx, &cff->global_index, gid);
 			end = index_get(ctx, &cff->global_index, gid+1);
-			execute_charstring(ctx, cff, &cff->base[offset], &cff->base[end]);
+
+			local_usage = get_font_locals(ctx, cff, gid, is_pdf_cidfont, &subr_bias);
+			execute_charstring(ctx, cff, &cff->base[offset], &cff->base[end], subr_bias, local_usage);
 			changed = 1;
 		}
 	}
@@ -1488,13 +1576,13 @@ get_charset_len(fz_context *ctx, cff_t *cff)
 }
 
 static void
-get_fdselect_len(fz_context *ctx, cff_t *cff)
+read_fdselect(fz_context *ctx, cff_t *cff)
 {
 	uint32_t fdselect_offset = cff->fdselect_offset;
 	const uint8_t *d = cff->base + fdselect_offset;
 	const uint8_t *d0 = d;
 	uint8_t fmt;
-	uint32_t n;
+	uint16_t n, m, i, first, last, k;
 
 	if (fdselect_offset == 0)
 	{
@@ -1508,19 +1596,39 @@ get_fdselect_len(fz_context *ctx, cff_t *cff)
 	fmt = *d++;
 	n = cff->charstrings_index.count;
 
+	cff->gid_to_font = fz_calloc(ctx, n, sizeof(*cff->gid_to_font));
+
 	if (fmt == 0)
 	{
-		d += n;
+		for (i = 0; i < n; i++)
+		{
+			if (d >= cff->base + cff->len)
+				fz_throw(ctx, FZ_ERROR_FORMAT, "corrupt fdselect");
+			cff->gid_to_font[i] = d[0];
+			d++;
+		}
 	}
 	else if (fmt == 3)
 	{
 		if (d + 2 >= cff->base + cff->len)
 			fz_throw(ctx, FZ_ERROR_FORMAT, "corrupt fdselect");
-		n = get16(d);
-		d += 2 + 3*n;
-		if (d + 2 >= cff->base + cff->len)
-			fz_throw(ctx, FZ_ERROR_FORMAT, "corrupt fdselect");
+		m = get16(d);
 		d += 2;
+		if (m > cff->charstrings_index.count)
+			fz_throw(ctx, FZ_ERROR_FORMAT, "corrupt fdselect");
+
+		for (i = 0; i < m; i++)
+		{
+			if (d + 5 >= cff->base + cff->len)
+				fz_throw(ctx, FZ_ERROR_FORMAT, "corrupt fdselect");
+			first = get16(d);
+			last = get16(d + 3);
+			if (first >= cff->charstrings_index.count || last > cff->charstrings_index.count || first >= last)
+				fz_throw(ctx, FZ_ERROR_FORMAT, "corrupt fdselect");
+			for (k = first; k < last; k++)
+				cff->gid_to_font[k] = d[2];
+			d += 3;
+		}
 	}
 
 	cff->fdselect_len = (uint32_t)(d - d0);
@@ -1556,13 +1664,13 @@ load_charset_for_cidfont(fz_context *ctx, cff_t *cff)
 		for (i = 1; i < n;)
 		{
 			uint16_t first;
-			uint32_t nleft;
+			int32_t nleft;
 			if (d + 3 >= cff->base + cff->len)
 				fz_throw(ctx, FZ_ERROR_FORMAT, "corrupt charset");
 			first = get16(d);
 			nleft = d[2] + 1;
 			d += 3;
-			while (nleft--)
+			while (nleft-- && i < n)
 			{
 				cff->gid_to_cid[i++] = first++;
 			}
@@ -1570,16 +1678,16 @@ load_charset_for_cidfont(fz_context *ctx, cff_t *cff)
 	}
 	else if (fmt == 2)
 	{
-		for (i = 1; i < n; i++)
+		for (i = 1; i < n;)
 		{
 			uint16_t first;
-			uint32_t nleft;
+			int32_t nleft;
 			if (d + 4 >= cff->base + cff->len)
 				fz_throw(ctx, FZ_ERROR_FORMAT, "corrupt charset");
 			first = get16(d);
-			nleft = get16(d+2);
+			nleft = get16(d+2) + 1;
 			d += 4;
-			while (nleft--)
+			while (nleft-- && i < n)
 			{
 				cff->gid_to_cid[i++] = first++;
 			}
@@ -1645,7 +1753,7 @@ rewrite_fdarray(fz_context *ctx, cff_t *cff, uint32_t offset0)
 	uint16_t n = cff->fdarray_index.count;
 	uint32_t len = 0;
 	uint8_t os;
-	uint32_t offset;
+	size_t offset;
 
 	if (cff->fdarray == NULL)
 		fz_throw(ctx, FZ_ERROR_FORMAT, "Expected to rewrite an fdarray");
@@ -1666,12 +1774,20 @@ rewrite_fdarray(fz_context *ctx, cff_t *cff, uint32_t offset0)
 	{
 		assert(cff->fdarray[i].rewritten_dict->data[cff->fdarray[i].fixup] == 29);
 		assert(cff->fdarray[i].rewritten_dict->data[cff->fdarray[i].fixup+5] == 29);
-		put32(&cff->fdarray[i].rewritten_dict->data[cff->fdarray[i].fixup+1], cff->fdarray[i].len);
-		put32(&cff->fdarray[i].rewritten_dict->data[cff->fdarray[i].fixup+6], offset);
-		offset += cff->fdarray[i].len;
+		put32(&cff->fdarray[i].rewritten_dict->data[cff->fdarray[i].fixup+1], (uint32_t)cff->fdarray[i].rewritten_private->len);
+		put32(&cff->fdarray[i].rewritten_dict->data[cff->fdarray[i].fixup+6], (uint32_t)offset);
+		offset += cff->fdarray[i].rewritten_private->len;
+		if (cff->fdarray[i].local_subset)
+		{
+			offset += cff->fdarray[i].local_subset->len;
+		}
+		else
+		{
+			offset += 2;
+		}
 	}
 
-	return offset;
+	return (uint32_t)offset;
 }
 
 static void
@@ -1760,12 +1876,18 @@ read_top_dict(fz_context *ctx, cff_t *cff, int idx)
 	{
 		switch (k)
 		{
+		case DICT_OP_ROS:
+			cff->is_cidfont = 1;
+			break;
 		case DICT_OP_charset:
 			cff->charset_offset = dict_arg_int(ctx, &di, 0);
 			break;
 		case DICT_OP_Encoding:
 			cff->encoding_offset = dict_arg_int(ctx, &di, 0);
 			break;
+		case DICT_OP_CharstringType:
+			cff->charstring_type = 1;
+			break;
 		case DICT_OP_CharStrings:
 			cff->charstrings_index_offset = dict_arg_int(ctx, &di, 0);
 			break;
@@ -1900,7 +2022,7 @@ make_new_private_dict(fz_context *ctx, cff_t *cff)
 		{
 			/* Everything is in the DICT except for the local subr offset. Insert
 			 * that now. This is tricky, because what is the offset? It depends on
-			 * the size of he dict we are creating now, and the size of the dict
+			 * the size of the dict we are creating now, and the size of the dict
 			 * we are creating now depends on the size of the offset! */
 			/* Length so far */
 			len = fz_tell_output(ctx, out);
@@ -1956,6 +2078,8 @@ read_fdarray_and_privates(fz_context *ctx, cff_t *cff)
 	dict_operator k;
 	uint16_t i;
 	uint16_t n = cff->fdarray_index.count;
+	int subrs;
+	int64_t len;
 
 	cff->fdarray = fz_calloc(ctx, n, sizeof(*cff->fdarray));
 
@@ -1996,6 +2120,91 @@ read_fdarray_and_privates(fz_context *ctx, cff_t *cff)
 			fz_drop_output(ctx, out);
 		fz_catch(ctx)
 			fz_rethrow(ctx);
+
+
+		offset = cff->fdarray[i].offset;
+		end = cff->fdarray[i].offset + cff->fdarray[i].len;
+
+		fz_try(ctx)
+		{
+			cff->fdarray[i].rewritten_private = fz_new_buffer(ctx, 1024);
+
+			out = fz_new_output_with_buffer(ctx, cff->fdarray[i].rewritten_private);
+			cff->fdarray[i].local_index_offset = 0;
+
+			subrs = 0;
+
+			for (k = dict_init(ctx, &di, cff->base, cff->len, offset, end); dict_more(&di); k = dict_next(ctx, &di))
+			{
+				switch (k)
+				{
+				case DICT_OP_Subrs:
+					subrs = 1;
+					cff->fdarray[i].local_index_offset = dict_arg_int(ctx, &di, 0) + offset;
+					break;
+				default:
+					dict_write_args(ctx, out, &di);
+					break;
+				}
+			}
+
+			if (subrs != 0)
+			{
+				/* Everything is in the DICT except for the local subr offset. Insert
+				 * that now. This is tricky, because what is the offset? It depends on
+				 * the size of he dict we are creating now, and the size of the dict
+				 * we are creating now depends on the size of the offset! */
+				/* Length so far */
+				len = fz_tell_output(ctx, out);
+				/* We have to encode an offset, plus the Subrs token (19). Offset
+				 * can take up to 5 bytes. */
+				if (len+2 < 107)
+				{
+					/* We can code it with a single byte encoding */
+					len += 2;
+					fz_write_byte(ctx, out, len + 139);
+				}
+				else if (len+3 < 1131)
+				{
+					/* We can code it with a 2 byte encoding */
+					/* (b0-247) * 256 + b1 + 108 == len+3 */
+					len = len+3 - 108;
+					fz_write_byte(ctx, out, (len>>8) + 247);
+					fz_write_byte(ctx, out, len);
+				}
+				else if (len+4 < 32767)
+				{
+					/* We can code it with a 3 byte encoding */
+					len += 4;
+					fz_write_byte(ctx, out, 28);
+					fz_write_byte(ctx, out, len>>8);
+					fz_write_byte(ctx, out, len);
+				}
+				else
+				{
+					/* We can code it with a 5 byte encoding */
+					len += 5;
+					fz_write_byte(ctx, out, 29);
+					fz_write_byte(ctx, out, len>>24);
+					fz_write_byte(ctx, out, len>>16);
+					fz_write_byte(ctx, out, len>>8);
+					fz_write_byte(ctx, out, len);
+				}
+				fz_write_byte(ctx, out, DICT_OP_Subrs);
+			}
+
+			fz_close_output(ctx, out);
+		}
+		fz_always(ctx)
+			fz_drop_output(ctx, out);
+		fz_catch(ctx)
+			fz_rethrow(ctx);
+
+		if (cff->fdarray[i].local_index_offset != 0)
+		{
+			index_load(ctx, &cff->fdarray[i].local_index, cff->base, (uint32_t)cff->len, cff->fdarray[i].local_index_offset);
+			cff->fdarray[i].subr_bias = subr_bias(ctx, cff, cff->fdarray[i].local_index.count);
+		}
 	}
 }
 
@@ -2034,7 +2243,11 @@ output_fdarray(fz_context *ctx, fz_output *out, cff_t *cff)
 	/* Now we can write out the private dicts, unchanged from the original file. */
 	for (i = 0; i < n; i++)
 	{
-		fz_write_data(ctx, out, cff->base + cff->fdarray[i].offset, cff->fdarray[i].len);
+		fz_write_data(ctx, out, cff->fdarray[i].rewritten_private->data, cff->fdarray[i].rewritten_private->len);
+		if (cff->fdarray[i].local_subset)
+			fz_write_data(ctx, out, cff->fdarray[i].local_subset->data, cff->fdarray[i].local_subset->len);
+		else
+			fz_write_uint16_be(ctx, out, 0);
 	}
 }
 
@@ -2053,8 +2266,9 @@ cid_to_gid(fz_context *ctx, cff_t *cff, uint16_t cid)
 	return 0;
 }
 
+
 fz_buffer *
-fz_subset_cff_for_gids(fz_context *ctx, fz_buffer *orig, int *gids, int num_gids, int symbolic, int cidfont)
+fz_subset_cff_for_gids(fz_context *ctx, fz_buffer *orig, int *gids, int num_gids, int symbolic, int is_pdf_cidfont)
 {
 	cff_t cff = { 0 };
 	fz_buffer *newbuf = NULL;
@@ -2062,6 +2276,7 @@ fz_subset_cff_for_gids(fz_context *ctx, fz_buffer *orig, int *gids, int num_gids
 	size_t len;
 	fz_output *out = NULL;
 	int i;
+	uint16_t n, k;
 
 	fz_var(newbuf);
 	fz_var(out);
@@ -2078,7 +2293,6 @@ fz_subset_cff_for_gids(fz_context *ctx, fz_buffer *orig, int *gids, int num_gids
 		cff.len = len;
 
 		cff.symbolic = symbolic;
-		cff.cidfont = cidfont;
 
 		if (len < 4)
 			fz_throw(ctx, FZ_ERROR_FORMAT, "Truncated CFF");
@@ -2088,7 +2302,7 @@ fz_subset_cff_for_gids(fz_context *ctx, fz_buffer *orig, int *gids, int num_gids
 		cff.headersize = base[2];
 		cff.offsize = base[3];
 
-		if (cff.offsize < 0 || cff.offsize > 4)
+		if (cff.offsize > 4)
 			fz_throw(ctx, FZ_ERROR_FORMAT, "Invalid offsize in CFF");
 
 		if (len > UINT32_MAX)
@@ -2105,61 +2319,62 @@ fz_subset_cff_for_gids(fz_context *ctx, fz_buffer *orig, int *gids, int num_gids
 
 		/* Next the Global subr index */
 		index_load(ctx, &cff.global_index, base, (uint32_t)len, cff.global_index_offset);
-		if (cff.global_index.count < 1240)
-			cff.gsubr_bias = 107;
-		else if (cff.global_index.count < 33900)
-			cff.gsubr_bias = 1131;
-		else
-			cff.gsubr_bias = 32768;
+
+		/* Default value, possibly updated by top dict entries */
+		cff.charstring_type = 2;
 
 		/* CFF files can contain several fonts, but we only want the first one. */
 		read_top_dict(ctx, &cff, 0);
 
+		cff.gsubr_bias = subr_bias(ctx, &cff, cff.global_index.count);
+
 		if (cff.charstrings_index_offset == 0)
 			fz_throw(ctx, FZ_ERROR_FORMAT, "Missing charstrings table");
 
 		index_load(ctx, &cff.charstrings_index, base, (uint32_t)len, cff.charstrings_index_offset);
 		index_load(ctx, &cff.local_index, base, (uint32_t)len, cff.local_index_offset);
-		if (cff.local_index.count < 1240)
-			cff.subr_bias = 107;
-		else if (cff.local_index.count < 33900)
-			cff.subr_bias = 1131;
-		else
-			cff.subr_bias = 32768;
+		cff.subr_bias = subr_bias(ctx, &cff, cff.local_index.count);
 		index_load(ctx, &cff.fdarray_index, base, (uint32_t)len, cff.fdarray_index_offset);
 
-		/* Move our list of gids into our own storage. */
-		if (cidfont)
+		get_encoding_len(ctx, &cff);
+		get_charset_len(ctx, &cff);
+
+		if (is_pdf_cidfont && cff.is_cidfont)
 		{
-			/* For CIDFonts we are given CIDs here, not gids. Accordingly
-			 * we need to look them up in the charset */
+			read_fdselect(ctx, &cff);
+			read_fdarray_and_privates(ctx, &cff);
+		}
+
+		/* Move our list of gids into our own storage. */
+		if (is_pdf_cidfont && cff.is_cidfont)
+		{
+			/* For CIDFontType0 FontDescriptor with a CFF that uses CIDFont operators,
+			 * we are given CIDs here, not GIDs. Accordingly
+			 * we need to look them up in the CharSet.
+			 */
 			load_charset_for_cidfont(ctx, &cff);
 			for (i = 0; i < num_gids; i++)
 				usage_list_add(ctx, &cff.gids_to_keep, cid_to_gid(ctx, &cff, gids[i]));
 		}
 		else
 		{
+			/* For CIDFontType0 FontDescriptor with a CFF that DOES NOT use CIDFont operators,
+			 * and for Type1 FontDescriptors, we are given GIDs directly.
+			 */
 			for (i = 0; i < num_gids; i++)
 				usage_list_add(ctx, &cff.gids_to_keep, gids[i]);
 		}
 
-		get_encoding_len(ctx, &cff);
-		get_charset_len(ctx, &cff);
-
 		/* Scan charstrings. */
-		scan_charstrings(ctx, &cff);
+		scan_charstrings(ctx, &cff, is_pdf_cidfont);
 
 		/* Now subset the data. */
 		subset_charstrings(ctx, &cff);
+		if (is_pdf_cidfont && cff.is_cidfont)
+			subset_fdarray_locals(ctx, &cff);
 		subset_locals(ctx, &cff);
 		subset_globals(ctx, &cff);
 
-		if (cidfont)
-		{
-			get_fdselect_len(ctx, &cff);
-			read_fdarray_and_privates(ctx, &cff);
-		}
-
 		/* FIXME: cull the strings? */
 
 		/*	Now, rewrite the font.
@@ -2270,17 +2485,21 @@ fz_subset_cff_for_gids(fz_context *ctx, fz_buffer *orig, int *gids, int num_gids
 		fz_drop_buffer(ctx, cff.local_subset);
 		fz_drop_buffer(ctx, cff.global_subset);
 		fz_free(ctx, cff.gid_to_cid);
+		fz_free(ctx, cff.gid_to_font);
 		drop_usage_list(ctx, &cff.local_usage);
 		drop_usage_list(ctx, &cff.global_usage);
 		drop_usage_list(ctx, &cff.gids_to_keep);
 		drop_usage_list(ctx, &cff.extra_gids_to_keep);
 		if (cff.fdarray)
 		{
-			int n = cff.fdarray_index.count;
-			int i;
-
-			for (i = 0; i < n; i++)
-				fz_drop_buffer(ctx, cff.fdarray[i].rewritten_dict);
+			n = cff.fdarray_index.count;
+			for (k = 0; k < n; k++)
+			{
+				fz_drop_buffer(ctx, cff.fdarray[k].rewritten_dict);
+				fz_drop_buffer(ctx, cff.fdarray[k].rewritten_private);
+				fz_drop_buffer(ctx, cff.fdarray[k].local_subset);
+				drop_usage_list(ctx, &cff.fdarray[k].local_usage);
+			}
 			fz_free(ctx, cff.fdarray);
 		}
 		fz_free(ctx, cff.unpacked_charset);
